# -*- coding: utf-8 -*-
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import abc
from typing import Awaitable, Callable, Dict, Optional, Sequence, Union

from seqq.api_v1alpha import gapic_version as package_version

import google.auth  # type: ignore
import google.api_core
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account # type: ignore

from google.protobuf import empty_pb2  # type: ignore
from seqq.api_v1alpha.types import collection
from seqq.api_v1alpha.types import collection as sa_collection
from seqq.api_v1alpha.types import integration
from seqq.api_v1alpha.types import integration as sa_integration
from seqq.api_v1alpha.types import search
from seqq.api_v1alpha.types import search as sa_search
from seqq.api_v1alpha.types import sequence
from seqq.api_v1alpha.types import sequence as sa_sequence

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(gapic_version=package_version.__version__)


class SeqqServiceTransport(abc.ABC):
    """Abstract transport class for SeqqService."""

    AUTH_SCOPES = (
    )

    DEFAULT_HOST: str = 'api.seqq.io'
    def __init__(
            self, *,
            host: str = DEFAULT_HOST,
            credentials: Optional[ga_credentials.Credentials] = None,
            credentials_file: Optional[str] = None,
            scopes: Optional[Sequence[str]] = None,
            quota_project_id: Optional[str] = None,
            client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
            always_use_jwt_access: Optional[bool] = False,
            api_audience: Optional[str] = None,
            **kwargs,
            ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'api.seqq.io').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is mutually exclusive with credentials.
            scopes (Optional[Sequence[str]]): A list of scopes.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
        """

        scopes_kwargs = {"scopes": scopes, "default_scopes": self.AUTH_SCOPES}

        # Save the scopes.
        self._scopes = scopes

        # If no credentials are provided, then determine the appropriate
        # defaults.
        if credentials and credentials_file:
            raise core_exceptions.DuplicateCredentialArgs("'credentials_file' and 'credentials' are mutually exclusive")

        if credentials_file is not None:
            credentials, _ = google.auth.load_credentials_from_file(
                                credentials_file,
                                **scopes_kwargs,
                                quota_project_id=quota_project_id
                            )
        elif credentials is None:
            credentials, _ = google.auth.default(**scopes_kwargs, quota_project_id=quota_project_id)
            # Don't apply audience if the credentials file passed from user.
            if hasattr(credentials, "with_gdch_audience"):
                credentials = credentials.with_gdch_audience(api_audience if api_audience else host)

        # If the credentials are service account credentials, then always try to use self signed JWT.
        if always_use_jwt_access and isinstance(credentials, service_account.Credentials) and hasattr(service_account.Credentials, "with_always_use_jwt_access"):
            credentials = credentials.with_always_use_jwt_access(True)

        # Save the credentials.
        self._credentials = credentials

        # Save the hostname. Default to port 443 (HTTPS) if none is specified.
        if ':' not in host:
            host += ':443'
        self._host = host

    @property
    def host(self):
        return self._host

    def _prep_wrapped_messages(self, client_info):
        # Precompute the wrapped methods.
        self._wrapped_methods = {
            self.create_collection: gapic_v1.method.wrap_method(
                self.create_collection,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_collection: gapic_v1.method.wrap_method(
                self.get_collection,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_collections: gapic_v1.method.wrap_method(
                self.list_collections,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_collection: gapic_v1.method.wrap_method(
                self.delete_collection,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_sequence: gapic_v1.method.wrap_method(
                self.create_sequence,
                default_timeout=None,
                client_info=client_info,
            ),
            self.batch_create_sequences: gapic_v1.method.wrap_method(
                self.batch_create_sequences,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_sequences_from_file: gapic_v1.method.wrap_method(
                self.create_sequences_from_file,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_sequence: gapic_v1.method.wrap_method(
                self.get_sequence,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_sequences: gapic_v1.method.wrap_method(
                self.list_sequences,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_sequence: gapic_v1.method.wrap_method(
                self.delete_sequence,
                default_timeout=None,
                client_info=client_info,
            ),
            self.start_search: gapic_v1.method.wrap_method(
                self.start_search,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_search: gapic_v1.method.wrap_method(
                self.get_search,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_searches: gapic_v1.method.wrap_method(
                self.list_searches,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_search: gapic_v1.method.wrap_method(
                self.delete_search,
                default_timeout=None,
                client_info=client_info,
            ),
            self.create_integration: gapic_v1.method.wrap_method(
                self.create_integration,
                default_timeout=None,
                client_info=client_info,
            ),
            self.get_integration: gapic_v1.method.wrap_method(
                self.get_integration,
                default_timeout=None,
                client_info=client_info,
            ),
            self.update_integration: gapic_v1.method.wrap_method(
                self.update_integration,
                default_timeout=None,
                client_info=client_info,
            ),
            self.list_integrations: gapic_v1.method.wrap_method(
                self.list_integrations,
                default_timeout=None,
                client_info=client_info,
            ),
            self.delete_integration: gapic_v1.method.wrap_method(
                self.delete_integration,
                default_timeout=None,
                client_info=client_info,
            ),
         }

    def close(self):
        """Closes resources associated with the transport.

       .. warning::
            Only call this method if the transport is NOT shared
            with other clients - this may cause errors in other clients!
        """
        raise NotImplementedError()

    @property
    def create_collection(self) -> Callable[
            [sa_collection.CreateCollectionRequest],
            Union[
                sa_collection.Collection,
                Awaitable[sa_collection.Collection]
            ]]:
        raise NotImplementedError()

    @property
    def get_collection(self) -> Callable[
            [collection.GetCollectionRequest],
            Union[
                collection.Collection,
                Awaitable[collection.Collection]
            ]]:
        raise NotImplementedError()

    @property
    def list_collections(self) -> Callable[
            [collection.ListCollectionsRequest],
            Union[
                collection.ListCollectionsResponse,
                Awaitable[collection.ListCollectionsResponse]
            ]]:
        raise NotImplementedError()

    @property
    def delete_collection(self) -> Callable[
            [collection.DeleteCollectionRequest],
            Union[
                empty_pb2.Empty,
                Awaitable[empty_pb2.Empty]
            ]]:
        raise NotImplementedError()

    @property
    def create_sequence(self) -> Callable[
            [sa_sequence.CreateSequenceRequest],
            Union[
                sa_sequence.Sequence,
                Awaitable[sa_sequence.Sequence]
            ]]:
        raise NotImplementedError()

    @property
    def batch_create_sequences(self) -> Callable[
            [sequence.BatchCreateSequencesRequest],
            Union[
                sequence.BatchCreateSequencesResponse,
                Awaitable[sequence.BatchCreateSequencesResponse]
            ]]:
        raise NotImplementedError()

    @property
    def create_sequences_from_file(self) -> Callable[
            [sequence.CreateSequencesFromFileRequest],
            Union[
                sequence.CreateSequencesFromFileResponse,
                Awaitable[sequence.CreateSequencesFromFileResponse]
            ]]:
        raise NotImplementedError()

    @property
    def get_sequence(self) -> Callable[
            [sequence.GetSequenceRequest],
            Union[
                sequence.Sequence,
                Awaitable[sequence.Sequence]
            ]]:
        raise NotImplementedError()

    @property
    def list_sequences(self) -> Callable[
            [sequence.ListSequencesRequest],
            Union[
                sequence.ListSequencesResponse,
                Awaitable[sequence.ListSequencesResponse]
            ]]:
        raise NotImplementedError()

    @property
    def delete_sequence(self) -> Callable[
            [sequence.DeleteSequenceRequest],
            Union[
                empty_pb2.Empty,
                Awaitable[empty_pb2.Empty]
            ]]:
        raise NotImplementedError()

    @property
    def start_search(self) -> Callable[
            [sa_search.StartSearchRequest],
            Union[
                sa_search.Search,
                Awaitable[sa_search.Search]
            ]]:
        raise NotImplementedError()

    @property
    def get_search(self) -> Callable[
            [search.GetSearchRequest],
            Union[
                search.Search,
                Awaitable[search.Search]
            ]]:
        raise NotImplementedError()

    @property
    def list_searches(self) -> Callable[
            [search.ListSearchesRequest],
            Union[
                search.ListSearchesResponse,
                Awaitable[search.ListSearchesResponse]
            ]]:
        raise NotImplementedError()

    @property
    def delete_search(self) -> Callable[
            [search.DeleteSearchRequest],
            Union[
                empty_pb2.Empty,
                Awaitable[empty_pb2.Empty]
            ]]:
        raise NotImplementedError()

    @property
    def create_integration(self) -> Callable[
            [sa_integration.CreateIntegrationRequest],
            Union[
                sa_integration.Integration,
                Awaitable[sa_integration.Integration]
            ]]:
        raise NotImplementedError()

    @property
    def get_integration(self) -> Callable[
            [integration.GetIntegrationRequest],
            Union[
                integration.Integration,
                Awaitable[integration.Integration]
            ]]:
        raise NotImplementedError()

    @property
    def update_integration(self) -> Callable[
            [sa_integration.UpdateIntegrationRequest],
            Union[
                sa_integration.Integration,
                Awaitable[sa_integration.Integration]
            ]]:
        raise NotImplementedError()

    @property
    def list_integrations(self) -> Callable[
            [integration.ListIntegrationsRequest],
            Union[
                integration.ListIntegrationsResponse,
                Awaitable[integration.ListIntegrationsResponse]
            ]]:
        raise NotImplementedError()

    @property
    def delete_integration(self) -> Callable[
            [integration.DeleteIntegrationRequest],
            Union[
                empty_pb2.Empty,
                Awaitable[empty_pb2.Empty]
            ]]:
        raise NotImplementedError()

    @property
    def kind(self) -> str:
        raise NotImplementedError()


__all__ = (
    'SeqqServiceTransport',
)
